From 0481586bfce18615c7e5e1c32f307152eaea8df9 Mon Sep 17 00:00:00 2001
From: Justin Miller <justin@runicgames.com>
Date: Wed, 9 Feb 2011 10:50:55 -0800
Subject: [PATCH] Fixed out of memory error in Git.pm cat_blob

cat_blob was doing small (1KB) reads in a loop, but still
just building up the entire buffer into memory without doing
anything with it but writing it out. This was causing out of
memory errors for files above around 380MB on Windows with
msysgit. Now it reads up to 1KB, and writes that out without
holding onto it. This should also make it faster, as every
read was reallocating the buffer to add to the end.
---
 perl/Git.pm |   15 +++++++--------
 1 files changed, 7 insertions(+), 8 deletions(-)

diff --git a/perl/Git.pm b/perl/Git.pm
index 205e48a..cb0e50b 100644
--- a/perl/Git.pm
+++ b/perl/Git.pm
@@ -887,21 +887,25 @@ sub cat_blob {
 	}
 
 	my $size = $1;
-
-	my $blob;
 	my $bytesRead = 0;
 
 	while (1) {
 		my $bytesLeft = $size - $bytesRead;
 		last unless $bytesLeft;
 
+		my $blob;
 		my $bytesToRead = $bytesLeft < 1024 ? $bytesLeft : 1024;
-		my $read = read($in, $blob, $bytesToRead, $bytesRead);
+		my $read = read($in, $blob, $bytesToRead);
 		unless (defined($read)) {
 			$self->_close_cat_blob();
 			throw Error::Simple("in pipe went bad");
 		}
 
+		unless (print $fh $blob) {
+			$self->_close_cat_blob();
+			throw Error::Simple("couldn't write to passed in filehandle");
+		}
+
 		$bytesRead += $read;
 	}
 
@@ -917,11 +921,6 @@ sub cat_blob {
 		throw Error::Simple("didn't find newline after blob");
 	}
 
-	unless (print $fh $blob) {
-		$self->_close_cat_blob();
-		throw Error::Simple("couldn't write to passed in filehandle");
-	}
-
 	return $size;
 }
 
-- 
1.7.3.1.msysgit.0

